@@ -28,7 +28,9 @@ func ManInfoLinter(ctx context.Context, _ *config.Configuration, pkgname string,
 		return nil
 	}
 	return AllPaths(ctx, pkgname, fsys,
-		func(path string) bool { return ManRegex.MatchString(path) || InfoRegex.MatchString(path) },
+		func(path string, d fs.DirEntry) bool {
+			return !d.IsDir() && (ManRegex.MatchString(path) || InfoRegex.MatchString(path))
+		},
 		func(pkgname string, paths []string) string {
 			fileWord := "file"
 			if len(paths) > 1 {
