@@ -25,7 +25,7 @@ import (
 
 func VarEmptyLinter(ctx context.Context, _ *config.Configuration, pkgname string, fsys fs.FS) error {
 	return AllPaths(ctx, pkgname, fsys,
-		func(path string) bool { return strings.HasPrefix(path, "var/empty/") },
+		func(path string, d fs.DirEntry) bool { return !d.IsDir() && strings.HasPrefix(path, "var/empty/") },
 		func(pkgname string, paths []string) string { return fmt.Sprintf("%s writes to /var/empty", pkgname) },
 	)
 }
